@@ -414,7 +414,7 @@ class HistorySearchThread(QThread):
                         text = hglib.tounicode(text)
                         text = Qt.escape(text)
                         text = '<b>%s</b> <span>%s</span>' % (addremove, text)
-                        row = [fname, rev, line, user, text]
+                        row = [fname, int(rev), int(line), user, text]
                         emitrow(row)
                     except ValueError:
                         pass
