@@ -15,7 +15,7 @@ import tempfile
 import re
 import weakref
 
-from mercurial import extensions, error, util
+from mercurial import commands, extensions, error, util
 
 from tortoisehg.util import hglib, paths
 from tortoisehg.hgqt.i18n import _
@@ -89,15 +89,8 @@ def openlocalurl(path):
     return QDesktopServices.openUrl(qurl)
 
 def openfiles(repo, files, parent=None):
-    if os.name == 'nt':
-        editor = 'start'
-    elif os.name == 'mac':
-        editor = 'open'
-    elif os.name == 'posix':
-        editor = 'xdg-open'
-    else:
-        editor = None
-    editfiles(repo, files, editor=editor)
+    for filename in files:
+        openlocalurl(repo.wjoin(filename))
 
 def editfiles(repo, files, lineno=None, search=None, parent=None, editor=None):
     if len(files) == 1:
@@ -166,6 +159,31 @@ def editfiles(repo, files, lineno=None, search=None, parent=None, editor=None):
                               hglib.tounicode(str(e))))
     return False
 
+def savefiles(repo, files, rev, parent=None):
+    for curfile in files:
+        wfile = util.localpath(curfile)
+        wfile, ext = os.path.splitext(os.path.basename(wfile))
+        if wfile:
+            filename = "%s@%d%s" % (wfile, rev, ext)
+        else:
+            filename = "%s@%d" % (ext, rev)
+        result = QFileDialog.getSaveFileName(parent=parent,
+                                             caption=_("Save file to"),
+                                             directory=filename) 
+        if not result:
+            continue
+        cwd = os.getcwd()
+        try:
+            os.chdir(repo.root)
+            try:
+                commands.cat(repo.ui, repo, curfile, rev=rev,
+                             output=hglib.fromunicode(result))
+            except (util.Abort, IOError), e:
+                QMessageBox.critical(self, _('Unable to save file'),
+                                     hglib.tounicode(str(e)))
+        finally:
+            os.chdir(cwd)
+
 _user_shell = None
 def openshell(root, reponame):
     if not os.path.exists(root):
@@ -1027,3 +1045,12 @@ def getCurrentUsername(widget, repo, opts=None):
     except error.Abort:
         return None
 
+def getTextInput(parent, title, label, mode=QLineEdit.Normal, text='',
+  flags=Qt.WindowFlags()):
+    # the flags argument is supported under Qt 4.6, but probably with
+    # a different name (see issue 252), so we simply call everything
+    # positionally
+    return QInputDialog.getText(parent, title, label, mode, text,
+      Qt.CustomizeWindowHint | Qt.WindowTitleHint |
+      Qt.WindowCloseButtonHint | flags)
+
