@@ -14,8 +14,11 @@
 # this program; if not, write to the Free Software Foundation, Inc.,
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
+import binascii
+
 from mercurial import util, error
 from mercurial.util import propertycache
+from mercurial.context import workingctx
 
 from tortoisehg.util import hglib
 from tortoisehg.hgqt.graph import Graph
@@ -34,7 +37,7 @@ mqpatchmimetype = 'application/thg-mqunappliedpatch'
 
 # TODO: Remove these two when we adopt GTK author color scheme
 COLORS = [ "blue", "darkgreen", "red", "green", "darkblue", "purple",
-           "cyan", Qt.darkYellow, "magenta", "darkred", "darkmagenta",
+           "dodgerblue", Qt.darkYellow, "magenta", "darkred", "darkmagenta",
            "darkcyan", "gray", ]
 COLORS = [str(QColor(x).name()) for x in COLORS]
 
@@ -50,6 +53,8 @@ COLUMNHEADERS = (
     ('LocalTime', _('Local Time', 'column header')),
     ('UTCTime', _('UTC Time', 'column header')),
     ('Changes', _('Changes', 'column header')),
+    ('Converted', _('Converted From', 'column header')),
+    ('Phase', _('Phase', 'column header')),
     )
 
 UNAPPLIED_PATCH_COLOR = '#999999'
@@ -76,7 +81,7 @@ class HgRepoListModel(QAbstractTableModel):
     _allcolumns = tuple(h[0] for h in COLUMNHEADERS)
     _allcolnames = dict(COLUMNHEADERS)
 
-    _columns = ('Graph', 'Rev', 'Branch', 'Description', 'Author', 'Age', 'Tags',)
+    _columns = ('Graph', 'Rev', 'Branch', 'Description', 'Author', 'Age', 'Tags', 'Phase',)
     _stretchs = {'Description': 1, }
     _mqtags = ('qbase', 'qtip', 'qparent')
 
@@ -103,9 +108,22 @@ class HgRepoListModel(QAbstractTableModel):
         self._branch_colors = {}
 
         if repo:
+            self.initBranchColors()
             self.reloadConfig()
             self.updateColumns()
             self.setBranch(branch)
+    
+    def initBranchColors(self):
+        # Set all the branch colors once on a fixed order,
+        # which should make the branch colors more stable
+        
+        # Always assign the first color to the default branch
+        self.namedbranch_color('default')
+        # Then assign colors to all branches in alphabetical order
+        # Note that re-assigning the color to the default branch
+        # is not expensive
+        for branch in sorted(self.repo.branchtags().keys()):
+            self.namedbranch_color(branch)
 
     def setBranch(self, branch=None, allparents=True):
         self.filterbranch = branch  # unicode
@@ -465,6 +483,8 @@ class HgRepoListModel(QAbstractTableModel):
         dragflags = Qt.ItemFlags(0)
         if ctx.thgmqunappliedpatch():
             dragflags = Qt.ItemIsDragEnabled | Qt.ItemIsDropEnabled
+        if isinstance(ctx, workingctx):
+            dragflags |= Qt.ItemIsDropEnabled
         if not self.revset:
             return Qt.ItemIsSelectable | Qt.ItemIsEnabled | dragflags
         if ctx.rev() not in self.revset:
@@ -641,6 +661,31 @@ class HgRepoListModel(QAbstractTableModel):
             addtotal(R, 'log.removed')
         return ''.join(changes)
 
+    def getconv(self, ctx, gnode):
+        if ctx.rev() is not None:
+            cvt = ctx.extra().get('convert_revision', '')
+            if cvt:
+                if cvt.startswith('svn:'):
+                    return cvt.split('@')[-1]
+                if len(cvt) == 40:
+                    try:
+                        binascii.unhexlify(cvt)
+                        return cvt[:12]
+                    except TypeError:
+                        pass
+            cvt = extra.get('p4', '')
+            if cvt:
+                return cvt
+        return ''
+
+    def getphase(self, ctx, gnode):
+        if ctx.rev() is None:
+            return ''
+        try:
+            return ctx.phasestr()
+        except:
+            return 'draft'
+
     _columnmap = {
         'Rev':      getrev,
         'Node':     lambda self, ctx, gnode: str(ctx),
@@ -654,4 +699,6 @@ class HgRepoListModel(QAbstractTableModel):
         'LocalTime':lambda self, ctx, gnode: hglib.displaytime(ctx.date()),
         'UTCTime':  lambda self, ctx, gnode: hglib.utctime(ctx.date()),
         'Changes':  getchanges,
+        'Converted': getconv,
+        'Phase':    getphase,
     }
